/*
 * blacklist-webclient.c - suid binary for blacklist
 * Copyright (C) 2004,2005 Ivan Ristic <ivanr@webkreator.com>
 * Copyright (C) 2005 Colm MacCarthaigh <colm.maccarthaigh@heanet.ie>
 *
 * $Id: blacklist-webclient.c,v 1.3 2005/09/22 20:50:19 ivanr Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 
 *
 * This binary is an interface between mod_security and the
 * blacklist script. It must be run as root so "chown root blacklist_webclient"
 * and "chmod u+s blacklist_webclient" are necessary. It will pick up the
 * REMOTE_ADDR from the environment and tell the blacklist
 * script to ban that address.
 *
 * As always, use the script with caution. You should note that it
 * is not possible to use this script directly if you are using
 * suEXEC (as it refuses to work with suid scripts such as this one).
 * Never allow the script to be used by people who don't already
 * have root privileges on the system.
 *
 * Warning: If you are executing external binaries based on user actions
 * (i.e. a request made to the web server) you should be aware that
 * there is a danger of a DoS. For example, if someone makes 100
 * HTTP requests every second, and every request results in binary
 * execution, depending on what these binaries do your server may
 * be overwhelmed. A much better blacklisting strategy, for example,
 * is to write IP addresses you want to ban to a text file, and call
 * blacklist from cron on every minute to process the list.
 *
 */

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#define USAGE "Usage: blacklist_webclient [IP address [duration]]"
#define BLACKLIST "/sbin/blacklist"
#define DURATION 3600

int is_ip_valid(char *ip) {
    /* accept only digits, dots, colons and hex */
    while(*ip != 0) {
        if (*ip == '.' || *ip == ':' || (*ip >= 'a' && *ip <= 'f') ||
            (*ip >= 'A' && *ip <= 'F') || isdigit(*ip))
            ip++;
         else
             return 0;
    }
    return 1;
}

int main(int argc, char **argv) {
    char buf[64];
    int duration = DURATION;
    char *ip = NULL;

    if (argc > 1) {
        ip = argv[1];
    }
    
    if (argc > 2) {
        duration = atoi(argv[2]);
    }
    
    if (argc > 3) {
        puts(USAGE);
        exit(1);
    }
    
    /* try to get the IP address from the environment */
    if (ip == NULL) ip = getenv("REMOTE_ADDR");
    
    if (ip == NULL) {
        puts(USAGE);
        puts("Error: No REMOTE_ADDR variable found");
        return;
    }
    
    if (!is_ip_valid(ip)) {
        puts(USAGE);
        printf("Error: Invalid IP address: %s", ip);
        return;
    }
    
    if (duration <= 0) {
        puts(USAGE);
        printf("Error: Invalid duration: %i", duration);
        return;
    }

    snprintf(buf, sizeof(buf) - 1, "%i", duration);    
        
    setuid(0);
    seteuid(0);
    execle(BLACKLIST, BLACKLIST, "block", ip, buf, NULL, NULL);

     /* If we're still running, there has been an error */
    printf("Error: could not execute %s block %s %s\n", BLACKLIST, ip, buf);
     
    return 0;
}
